import _objectSpread from "@babel/runtime/helpers/objectSpread";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import { ScriptTarget, ModuleResolutionKind, getPreEmitDiagnostics, flattenDiagnosticMessageText, ModuleKind } from 'typescript';
import plugin from "@start/plugin"; // https://github.com/Microsoft/TypeScript/wiki/Using-the-Compiler-API

export default ((outDirRelative, userOptions) => plugin('typescriptGenerate',
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* ({
    files,
    logFile
  }) {
    const _ref2 = yield import('typescript'),
          createProgram = _ref2.createProgram;

    const path = yield import('path');

    const options = _objectSpread({
      allowSyntheticDefaultImports: true,
      esModuleInterop: true,
      moduleResolution: ModuleResolutionKind.NodeJs,
      target: ScriptTarget.ESNext,
      module: ModuleKind.ESNext
    }, userOptions, {
      declarationDir: path.resolve(outDirRelative),
      declaration: true,
      emitDeclarationOnly: true
    });

    const filePaths = files.map(file => file.path);
    filePaths.forEach(logFile);
    const program = createProgram(filePaths, options);
    const emitResult = program.emit();
    const allDiagnostics = getPreEmitDiagnostics(program).concat(emitResult.diagnostics);
    allDiagnostics.forEach(diagnostic => {
      if (diagnostic.file) {
        const _diagnostic$file$getL = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start),
              line = _diagnostic$file$getL.line,
              character = _diagnostic$file$getL.character;

        const message = flattenDiagnosticMessageText(diagnostic.messageText, '\n');
        console.log(`${diagnostic.file.fileName} (${line + 1},${character + 1}): ${message}`);
      } else {
        console.log(`${flattenDiagnosticMessageText(diagnostic.messageText, '\n')}`);
      }
    });
    return {
      files
    };
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
}()));